/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.fleet.gpssimulator;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.CoordinateSequence;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.impl.CoordinateArraySequence;
import nl.cloudfarming.client.fleet.model.gps.GPSTracker;

/**
 * This class simulates the functionality of a GPS tracker by simulating driving
 * around in circles.
 *
 * @author Wytse Visser
 */
public final class GPSSimulator extends GPSTracker {

    private double timeSinceEnabled = 0;
    
    @Override
    public Point getPosition() {
        final double totalCircleTime = 120;
        final double angularVelocity = 0.008 / (totalCircleTime * UPDATES_PER_SECOND);
        
        timeSinceEnabled += 1.0 / UPDATES_PER_SECOND;
        double longitudeDifference = (Math.sin(Math.PI * (timeSinceEnabled / totalCircleTime)) * angularVelocity);
        double longitude = getMachine().getLongitude() + longitudeDifference;
        
        double latitudeDifference = (Math.cos(Math.PI * (timeSinceEnabled / totalCircleTime)) * angularVelocity);
        double latitude = getMachine().getLatitude() + latitudeDifference;
        
        Coordinate[] coordinates = new Coordinate[]{new Coordinate(longitude, latitude)};
        CoordinateSequence coordSequence = new CoordinateArraySequence(coordinates);
        return new Point(coordSequence, new GeometryFactory());
    }
}
